{-# LANGUAGE OverloadedStrings #-}
{-# LANGUAGE TemplateHaskell #-}

module Data.Aeson.TickLeiste
  ( KnownPlayer (..),
    UnknownPlayer (..),
    JSONRequest (..),
    JSONEvent (..),
  )
where

import Data.Aeson
import Data.Aeson.TH
import qualified Data.Text as T
import Data.TickLeiste
import qualified Data.UUID as U

$(deriveJSON defaultOptions {sumEncoding = defaultTaggedObject {tagFieldName = "tickType", contentsFieldName = "tickValue"}} ''Tick)

data KnownPlayer = KnownPlayer {kplayerUUID :: U.UUID, kplayerName :: Maybe T.Text}
  deriving (Show)

$(deriveJSON defaultOptions {fieldLabelModifier = drop 1} ''KnownPlayer)

data UnknownPlayer = UnknownPlayer {uplayerName :: T.Text}
  deriving (Show)

$(deriveJSON defaultOptions {fieldLabelModifier = drop 1} ''UnknownPlayer)

data JSONRequest
  = SetPlayerTickR {sptRplayer :: KnownPlayer, sptRtick :: Tick}
  | AddPladerTickR {aptRplayer :: UnknownPlayer, aptRtick :: Tick}
  | InitializeTickLeisteR {itlRtickLeiste :: [(Tick, [UnknownPlayer])]}
  | TickLeisteR
  | ChangeNameR {cnaRplayer :: KnownPlayer}
  deriving (Show)

$(deriveJSON defaultOptions {sumEncoding = defaultTaggedObject {tagFieldName = "requestType"}, fieldLabelModifier = drop 4} ''JSONRequest)

data JSONEvent
  = SetPlayerTickE {sptEplayer :: KnownPlayer, sptEtick :: Tick}
  | InitializeTickLeisteE {itlEtickLeiste :: [(Tick, [KnownPlayer])]}
  | ChangeNameE {cnaEplayer :: KnownPlayer}
  deriving (Show)

$(deriveJSON defaultOptions {sumEncoding = defaultTaggedObject {tagFieldName = "eventType"}, fieldLabelModifier = drop 4} ''JSONEvent)
